# lastfmpris

a rust application to scrobble your currently playing song on last.fm with mpris

## limitations

unfortunately, because of how most mpris players work, the [Track_Id](https://specifications.freedesktop.org/mpris-spec/latest/Player_Interface.html#Simple-Type:Track_Id) (unique identifier for every song in the tracklist) object is very rarely supported, or, if supported, implemented incorrectly (looking at you, [Cider](https://cider.sh/)) meaning it is impossible to tell if the track has been played twice in a row. this means you cannot have succssive scrobbles of the same song, sorry! besides this, i'd say the program is feature complete.

## configuration

either run the program once and navigate to `.config/lastmpris/config.ini` or create the file and copy it from the `config.example.ini` file provided in this directory. once you do that, configure your authentication methods. the api keys are available [here](https://www.last.fm/login?next=/api/account/create)

you also have the option to define your mpris bus name, which will instead wait for your specified player instead of waiting for the first active player

## how to install

*if you're on a setup that uses nix, you can just use `nix build` or the provided devshell to build the application.*

1. the nix way

    simply just run it:

    ```sh
    nix run git+https://git.reidlab.pink/reidlab/lastfmpris.git
    ```

    or import it into your flake like so:

    ```nix
    lastfmpris.url = "git+https://git.reidlab.pink/reidlab/lastfmpris.git";
    # in your `homeManagerConfiguration`:
    imports = [
      inputs.lastfmpris.homeManagerModules.lastfmpris;
    ];
    ```

    and then use it as a service in your `homeManagerConfiguration`:

    ```nix
    services.lastfmpris = {
      enable = true;
      settings = {
        global = {
          # player_bus = "org.mpris.MediaPlayer2.mpv";
          username = "ILoveLastDotFm";
          password = builtins.readFile /etc/lastfm-password;
          api_key = builtins.readFile /etc/lastfm-api-key;
          api_secret = builtins.readFile /etc/lastfm-api-secret;
        }
      };
    };
    ```

2. the normal way

    first, clone repository:

    ```sh
    git clone https://git.reidlab.pink/reidlab/lastfmpris && cd lastfmpris
    ```

    second, obtain dependencies (this will vary on your package manager, here, alpine is used)

    ```sh
    sudo apk add install pkgconf openssl dbus
    ```

    finally, run/build

    ```sh
    cargo run
    ```

    ```sh
    cargo build --release
    ```